//
// Created by auric on 2/21/24.
//

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <nprint.h>
#include <stdbool.h>
#include <sanity.h>
#include "config.h"
#include "debug.h"

enum Flags {
    FLAG_INSTALL = 1 << 0,
    FLAG_UPDATE = 1 << 1,
    FLAG_SOURCE = 1 << 2,
    FLAG_UNINSTALL = 1 << 3,
    FLAG_LIST = 1 << 4,
    FLAG_SEARCH = 1 << 5,
    FLAG_FORCE = 1 << 6,
    FLAG_UPGRADE = 1 << 7,
};

void help() {
    nprint("E", "Usage: fex (OPT) (PACKAGES)");
}

int main(int argc, char** argv) {
    unsigned short flags = 0;
    char* packages[argc - 1];
    int pkgCount = 0;
    if (argc < 2) {
        help();
        return 1;
    }

    for(int i = 1; i < argc; i++) {
        if(argv[i][0] == '-') {
            for(int j = 1; argv[i][j] != '\0'; j++) {
                switch(argv[i][j]) {
                    case 'i': // install, 0
                        flags |= FLAG_INSTALL;
                        break;
                    case 'u': // update, 1
                        flags |= FLAG_UPDATE;
                        break;
                    case 'S': // source, 2
                        flags |= FLAG_SOURCE;
                        break;
                    case 'r': // remove, 3
                        flags |= FLAG_UNINSTALL;
                        break;
                    case 'l': // list, 4
                        flags |= FLAG_LIST;
                        break;
                    case 's': // search, 5
                        flags |= FLAG_SEARCH;
                        break;
                    case 'f': // force, 6
                        flags |= FLAG_FORCE;
                        break;
                    case 'c': // upgrade; don't ask why I'm using 'c', 7
                        flags |= FLAG_UPGRADE;
                        break;
                    default:
                        nprint("E", "Flag '%c' is unknown.", argv[i][j]);
                        return 1;
                }
            }
        } else {
            packages[pkgCount++] = argv[i];
        }
    }

    const int sysSanity = isSysSane();

    #ifdef debugEnabled
        DEBUG_PRINT("sysSanity: %d", sysSanity);
        unsigned int flag_values[] = {0b00000001, 0b00000010, 0b00000100, 0b00001000, 0b00010000, 0b00100000, 0b01000000, 0b10000000};
        char* flag_names[] = {"Install", "Update", "Source", "Uninstall", "List", "Search", "Force", "Upgrade"};
        int num_flags = sizeof(flag_values) / sizeof(flag_values[0]);
        for(int i = 0; i < num_flags; i++) {
            if (flags & flag_values[i]) {
                nprint("D", "%s flag is set.", flag_names[i]);
            } else {
                nprint("D", "%s flag is not set.", flag_names[i]);
            }
        }
        for(int i = 0; i < pkgCount; i++) {
            nprint("D", "Package %d: %s", i, packages[i]);
        }
    #endif
    return 0;
}